@@ -8,7 +8,9 @@ import time
 import pytz
 import pandas as pd
 import numpy as np
-from datetime import datetime, timedelta
+import subprocess
+from shutil import copy2
+from datetime import datetime
 from autotrader.emailing import emailing
 from autotrader.lib import autodata, environment_manager
 from autotrader.lib.read_yaml import read_yaml
@@ -51,7 +53,6 @@ class AutoTraderBot():
         self.broker_utils       = autotrader_attributes.broker_utils
         self.email_params       = autotrader_attributes.email_params
         self.notify             = autotrader_attributes.notify
-        self.validation_file    = autotrader_attributes.validation_file
         self.verbosity          = autotrader_attributes.verbosity
         self.order_summary_fp   = autotrader_attributes.order_summary_fp
         self.backtest_mode      = autotrader_attributes.backtest_mode
@@ -65,6 +66,8 @@ class AutoTraderBot():
         self.include_broker     = autotrader_attributes.include_broker
         self.check_data_alignment = autotrader_attributes.check_data_alignment
         self.allow_dancing_bears = autotrader_attributes.allow_dancing_bears
+        self.use_stream         = autotrader_attributes.use_stream
+        self.MTF_initialisation = autotrader_attributes.MTF_initialisation
         
         self.instrument         = instrument
         self.broker             = broker
@@ -72,7 +75,7 @@ class AutoTraderBot():
         # Unpack strategy parameters
         interval                = strategy_config["INTERVAL"]
         period                  = strategy_config["PERIOD"]
-        risk_pc                 = strategy_config["RISK_PC"]
+        risk_pc                 = strategy_config["RISK_PC"] if 'RISK_PC' in strategy_config else 0
         sizing                  = strategy_config["SIZING"] if 'SIZING' in strategy_config else 0
         params                  = strategy_config["PARAMETERS"]
         
@@ -103,26 +106,80 @@ class AutoTraderBot():
                                                              global_config,
                                                              self.feed)
         
-        self.get_data           = autodata.GetData(broker_config, self.allow_dancing_bears)
-        data, quote_data        = self._retrieve_data(instrument, self.feed)
+        # Start price streaming
+        if self.use_stream and self.backtest_mode is False:
+            # TODO - use count from strat config to set max candles
+            
+            # TODO - note that current implementation will only stream base
+            # interval (first granularity listed)
+            
+            # Check how many granularities were requested
+            if len(interval.split(',')) > 1:
+                base_interval = interval.split(',')[0]
+                self.base_interval = base_interval
+                self.MTF_intervals = interval.split(',')[1:]
+            else:
+                self.base_interval = None
+                self.MTF_intervals = []
+                
+            
+            if base_interval == 'tick' or base_interval == 'ticks':
+                stream_type = 'ticks'
+                streamfile = "{}_ticks.txt".format(instrument)
+            else:
+                stream_type = 'candles'
+                streamfile = "{0}{1}.txt".format(base_interval, instrument)
+            
+            # Start streaming price data
+            stream_script_path = os.path.join(self.home_dir, 'oandastream.py')
+            stream_process = subprocess.Popen(['nohup', 'python3', stream_script_path,
+                                               '-i', instrument, '-g', base_interval,
+                                               '-t', stream_type])
+            
+            # Construct stream filename
+            streamfile_copy = 'copy_of_' + streamfile
+            abs_streamfile = os.path.join(self.home_dir, 'price_data', streamfile)
+            self.abs_streamfile_copy = os.path.join(self.home_dir, 'price_data', streamfile_copy)
+            print("Streaming price data to {} (PID: {}).".format(streamfile, 
+                                                                 stream_process.pid))
+            
+            # Set self.data to use streamfile 
+            self.data_file = abs_streamfile          
+            
+            # Sleep for 1 sec to allow stream to start and create file
+            time.sleep(1)
+            
         
-        # instantiate strategy
-        # my_strat = strategy(params, data, instrument)
+        if self.MTF_initialisation:
+            # Only retrieve MTF data once upon initialisation 
+            self.MTF_data = None
+        
+        self.get_data = autodata.GetData(broker_config, self.allow_dancing_bears)
+        data, quote_data, MTF_data = self._retrieve_data(instrument, self.feed)
+        
+        if MTF_data is None:
+            strat_data = data
+        else:
+            strat_data = MTF_data
         
         if self.include_broker:
-            # my_strat.broker = self.broker
-            # my_strat.broker_utils = self.broker_utils
-            my_strat = strategy(params, data, instrument, self.broker, self.broker_utils)
+            my_strat = strategy(params, strat_data, instrument, self.broker, self.broker_utils)
         else:
-            my_strat = strategy(params, data, instrument)
+            my_strat = strategy(params, strat_data, instrument)
             
-                
+        # Assign strategy
         self.strategy           = my_strat
         self.data               = data
         self.quote_data         = quote_data
         
         self.latest_orders      = []
         
+        # Assign strategy attributes for development purposes
+        if self.backtest_mode:
+            self.strategy._backtesting = True
+        if interval.split(',')[0] == 'tick':
+            self.strategy._tick_data = True
+        
         
         if int(self.verbosity) > 0:
                 print("AutoTraderBot assigned to analyse {}".format(instrument),
@@ -134,15 +191,17 @@ class AutoTraderBot():
         Method to update strategy with latest data. Called by the bot manager.
         '''
         
-        print("Updating strategy with latest data.")
-        
-        # TODO - add option to get dancing bears in autodata
-        
         # Download new data
-        new_data, _ = self._retrieve_data(self.instrument, self.feed)
+        new_data, _, MTF_data = self._retrieve_data(self.instrument, self.feed)
+        
+        # Check for MTF_data
+        if MTF_data is None:
+            strat_data = new_data
+        else:
+            strat_data = MTF_data
         
         # Update strategy with new data
-        self.strategy.initialise_strategy(new_data)
+        self.strategy.initialise_strategy(strat_data)
         
     
     def _retrieve_data(self, instrument, feed):
@@ -155,28 +214,12 @@ class AutoTraderBot():
         price_data_path = os.path.join(self.home_dir, 'price_data')
         
         if self.backtest_mode is True:
-            # Running in backtest mode
+            ' ~~~~~~~~~~~~~~~~~~~ Running in backtest mode ~~~~~~~~~~~~~~~~~~ '
             self.get_data.base_currency = self.base_currency
             
             from_date       = self.data_start
             to_date         = self.data_end
             
-            if self.validation_file is not None:
-                # Extract instrument-specific trade history as trade summary and trade period
-                livetrade_history = self.livetrade_history
-                formatted_instrument = instrument[:3] + "/" + instrument[-3:]
-                raw_livetrade_summary = livetrade_history[livetrade_history.Instrument == formatted_instrument] # FOR OANDA
-                from_date           = pd.to_datetime(raw_livetrade_summary.Date.values)[0]
-                to_date             = pd.to_datetime(raw_livetrade_summary.Date.values)[-1]
-                
-                self.raw_livetrade_summary = raw_livetrade_summary
-                
-                # Modify from date to improve backtest
-                from_date = from_date - period*timedelta(seconds = self.granularity_to_seconds(interval))
-                
-                # Modify starting balance
-                self.broker.portfolio_balance = raw_livetrade_summary.Balance.values[np.isfinite(raw_livetrade_summary.Balance.values)][0]
-                
             if self.data_file is not None:
                 custom_data_file        = self.data_file
                 custom_data_filepath    = os.path.join(price_data_path,
@@ -232,36 +275,148 @@ class AutoTraderBot():
                                            index_col = 0)
                         quote_data = pd.read_csv(historical_quote_data_file_path, 
                                                  index_col = 0)
+                    
+                    # TODO - add support of MTF for optimisation
+                    MTF_data = None
                         
                 else:
-                    data        = getattr(self.get_data, feed.lower())(instrument,
-                                                         granularity = interval,
-                                                         start_time = from_date,
-                                                         end_time = to_date)
-                    quote_data  = getattr(self.get_data, feed.lower() + '_quote_data')(data,
-                                                                    instrument,
-                                                                    interval,
-                                                                    from_date,
-                                                                    to_date)
+                    # Running in single backtest mode
+                    MTF_data = {}
+                    for granularity in interval.split(','):
+                        data        = getattr(self.get_data, feed.lower())(instrument,
+                                                             granularity = granularity,
+                                                             start_time = from_date,
+                                                             end_time = to_date)
+                        
+                        # Only get quote data for first granularity
+                        if granularity == interval.split(',')[0]:
+                            quote_data  = getattr(self.get_data, feed.lower() + '_quote_data')(data,
+                                                                            instrument,
+                                                                            granularity,
+                                                                            from_date,
+                                                                            to_date)
+                        
+                            data, quote_data    = self.broker_utils.check_dataframes(data, quote_data)
+                        
+                        MTF_data[granularity] = data
                     
-                    data, quote_data    = self.broker_utils.check_dataframes(data, quote_data)
+                    # Extract first dataset to use as base
+                    first_granularity = interval.split(',')[0]
+                    data = MTF_data[first_granularity]
+                    quote_data = quote_data
                 
+                if len(MTF_data) == 1:
+                    MTF_data = None
                 
                 if int(self.verbosity) > 1:
                     print("  Done.\n")
             
-            return data, quote_data
-            
-        else:
-            # Running in livetrade mode or scan mode
-            data = getattr(self.get_data, feed.lower())(instrument,
-                                                         granularity = interval,
-                                                         count=period)
-            if self.check_data_alignment:
-                data = self._verify_data_alignment(data, instrument, feed, period, 
-                                                  price_data_path)
+            return data, quote_data, MTF_data
         
-            return data, None
+        else:
+            ' ~~~~~~~~~~ Running in livetrade mode or scan mode ~~~~~~~~~~~~~ '
+            
+            if self.data_file is not None:
+                # Using price stream data
+                
+                custom_data_filepath = self.data_file
+                
+                # Make copy of file to prevent read-write errors
+                copy2(custom_data_filepath, self.abs_streamfile_copy)
+                
+                if int(self.verbosity) > 1:
+                    print("Using data file specified ({}).".format(custom_data_filepath))
+                
+                # Read datafile to get base interval data
+                data = pd.read_csv(self.abs_streamfile_copy, 
+                                   index_col = 0,
+                                   skipinitialspace=True)
+                
+                if len(data) > 0:
+                    # TODO - use count to replace 0 above
+                    data.index = pd.to_datetime(data.index)
+                    
+                    # Remove copied file
+                    os.remove(self.abs_streamfile_copy)
+                    
+                else:
+                    # Stream has not had enough time to write data yet, revert 
+                    # to downloading M1 data
+                    data = getattr(self.get_data, feed.lower())(instrument,
+                                                                granularity = 'M1',
+                                                                count = period)
+                    
+                    if self.check_data_alignment:
+                        data = self._verify_data_alignment(data, instrument, feed, period, 
+                                                           price_data_path)
+                
+                # Fetch MTF data
+                # TODO - wrap below in conditional if len(intervals) > 1
+                MTF_data = {self.base_interval: data}
+                
+                if self.MTF_initialisation:
+                    # Download MTF data for strategy initialisation only
+                    if self.MTF_data is None:
+                        # MTF_data has not been retrieved yet
+                        for granularity in self.MTF_intervals:
+                            data = getattr(self.get_data, feed.lower())(instrument,
+                                                                        granularity = granularity,
+                                                                        count=period)
+                            
+                            if self.check_data_alignment:
+                                data = self._verify_data_alignment(data, instrument, feed, period, 
+                                                                   price_data_path)
+                            
+                            MTF_data[granularity] = data
+                        
+                        self.MTF_data = MTF_data
+                        
+                    else:
+                        # MTF_data already exists, reuse
+                        MTF_data = self.MTF_data
+                        
+                        # Replace base_interval data with latest data
+                        MTF_data[self.base_interval] = data
+                        
+                else:
+                    # Download MTF data each update
+                    for granularity in self.MTF_intervals:
+                        data = getattr(self.get_data, feed.lower())(instrument,
+                                                                    granularity = granularity,
+                                                                    count=period)
+                        
+                        if self.check_data_alignment:
+                            data = self._verify_data_alignment(data, instrument, feed, period, 
+                                                               price_data_path)
+                        
+                        MTF_data[granularity] = data
+                        
+                        
+                if len(MTF_data) == 1:
+                    # There is only one timeframe of data
+                    MTF_data = None
+                    
+            else:            
+                # Running in periodic-download mode
+                MTF_data = {}
+                for granularity in interval.split(','):
+                    data = getattr(self.get_data, feed.lower())(instrument,
+                                                                granularity = granularity,
+                                                                count=period)
+                    
+                    if self.check_data_alignment:
+                        data = self._verify_data_alignment(data, instrument, feed, period, 
+                                                           price_data_path)
+                    
+                    MTF_data[granularity] = data
+                
+                first_granularity = interval.split(',')[0]
+                data = MTF_data[first_granularity]
+                
+                if len(MTF_data) == 1:
+                        MTF_data = None
+        
+            return data, None, MTF_data
 
 
     def _verify_data_alignment(self, data, instrument, feed, period, price_data_path):
@@ -536,17 +691,6 @@ class AutoTraderBot():
         trade_summary = self.broker_utils.trade_summary(self.instrument, self.broker.closed_positions)
         open_trade_summary = self.broker_utils.open_order_summary(self.instrument, self.broker.open_positions)
         cancelled_summary = self.broker_utils.cancelled_order_summary(self.instrument, self.broker.cancelled_orders)
-        
-        if self.validation_file is not None:
-            livetrade_summary = self.validation_utils.trade_summary(self.raw_livetrade_summary,
-                                                                    self.data,
-                                                                    self.strategy_params['granularity'])
-            final_balance_diff  = NAV[-1] - livetrade_summary.Balance.values[-1]
-            filled_live_orders  = livetrade_summary[livetrade_summary.Transaction == 'ORDER_FILL']
-            no_live_trades      = len(filled_live_orders)
-            self.livetrade_results = {'summary': livetrade_summary,
-                                      'final_balance_difference': final_balance_diff,
-                                      'no_live_trades': no_live_trades}
             
         backtest_dict = {}
         backtest_dict['data']           = self.data
