@@ -34,30 +34,76 @@ from autotrader.lib.bot_manager import ManageBot
 class AutoTrader():
     """
     AutoTrader: A Python-Based Development Platform For Automated Trading Systems.
+    ------------------------------------------------------------------------------
+    Website: https://kieran-mackle.github.io/AutoTrader/
     
-    Kieran Mackle
+    GitHub: https://github.com/kieran-mackle/AutoTrader
     
-    Version 0.4.0
+    Author: Kieran Mackle
     
+    Version: 0.4.x
 
     Attributes
     ----------
-    config_file : str
-        The strategy configuration file.
+    Note: many of the following attributes are set from the methods of AutoTrader.
     
+    feed : str 
+        The data feed to be used (eg. Yahoo, Oanda).
+                
     verbosity : int
-        The code verbosity.
+        The verbosity of AutoTrader (0, 1 or 2).
     
     notify : int
-         The emailing verbosity of the code.
+        The level of email notification (0, 1 or 2).
     
+    home_dir : str 
+        The project home directory.
+    
+    include_broker : bool 
+        Set to True to assign broker to strategy attributes.
+    
+    use_stream : bool 
+        Set to True to use price stream as data feed.
+    
+    detach_bot : bool 
+        Set to True to spawn new thread for each bot deployed.
+    
+    check_data_alignment : bool
+        Verify time of latest candle in data recieved against current time.
+    
+    allow_dancing_bears : bool
+        Allow incomplete candles to be passed to strategy.
+    
+    account_id : str
+        The brokerage account ID to use in this instance.
+    
+    environment : str
+        The trading environment of this instance.
+    
+    show_plot : bool
+        Automatically display plot of results.
+    
+    MTF_initialisation : bool
+        Only download mutliple time frame data when initialising the strategy, 
+        rather than every update.
+
 
     Methods
     -------
     run():
         Runs AutoTrader.
     
-    plot_backtest(bot=None, validation_file=None):
+    configure(feed='yahoo', verbosity=1, notify=0, home_dir=None,
+              include_broker=False, use_stream=False, detach_bot=False,
+              check_data_alignment=True, allow_dancing_bears=False,
+              account_id=None, environment='demo', show_plot=False,
+              MTF_initialisation=False):
+        Configures various run settings for AutoTrader.
+    
+    add_strategy(strategy_filename=None, strategy_dict=None)
+        Adds a strategy to AutoTrader. 
+    
+    plot_backtest(bot=None):
         Plots backtest results of an AutoTrader Bot.
     
     """
@@ -75,12 +121,13 @@ class AutoTrader():
         self.email_params   = None
         self.show_help      = None
         self.show_plot      = False
-        self.plot_validation_balance = True
         
         # Livetrade Parameters
         self.detach_bot     = False         # TODO - make this a strategy config option
         self.check_data_alignment = True
         self.allow_dancing_bears = False
+        self.use_stream     = False
+        self.MTF_initialisation = False
         
         # self.config         = None
         self.broker         = None
@@ -89,8 +136,10 @@ class AutoTrader():
         self.account_id     = None
         
         self.strategies     = {}
+        self._uninitiated_strat_files = []
+        self._uninitiated_strat_dicts = []
         self.feed           = 'yahoo'
-        self.include_broker = False
+        self.include_broker = False         # TODO - make this a strategy config option
         self.bots_deployed  = []
         
         # Backtesting Parameters
@@ -103,7 +152,6 @@ class AutoTrader():
         self.backtest_commission = None
         self.backtest_leverage = None
         self.backtest_base_currency = None
-        self.validation_file = None # TODO - rename to backtest_validation_file
         
         # Optimisation Parameters
         self.optimisation_config = None
@@ -117,15 +165,23 @@ class AutoTrader():
         self.scan_index = None
         self.scan_results = {}
         
-        # Make adjustments
-        if self.home_dir is None:
-            self.home_dir = os.getcwd()
-        
         
     def run(self):
         '''
         Run AutoTrader.
         '''
+        
+        # Define home_dir if undefined
+        if self.home_dir is None:
+            self.home_dir = os.getcwd()
+        
+        # Load uninitiated strategies
+        for strat_dict in self._uninitiated_strat_dicts:
+            self.add_strategy(strategy_dict=strat_dict)
+        for strat_config_file in self._uninitiated_strat_files:
+            self.add_strategy(strategy_filename=strat_config_file)
+        
+        # Print help
         if self.show_help is not None:
             printout.option_help(self.show_help)
         
@@ -178,10 +234,6 @@ class AutoTrader():
         ''' -------------------------------------------------------------- '''
         '''                         Load configuration                     '''
         ''' -------------------------------------------------------------- '''
-        if self.validation_file is not None:
-            livetrade_history   = pd.read_csv(self.validation_file, index_col = 0)
-            self.livetrade_history = livetrade_history.fillna(method='ffill')
-        
         # Construct broker config
         global_config_fp = os.path.join(self.home_dir, 'config', 'GLOBAL.yaml')
         if os.path.isfile(global_config_fp):
@@ -196,18 +248,6 @@ class AutoTrader():
             # Overwrite default account in global config
             broker_config['ACCOUNT_ID'] = self.account_id
         
-        # Get watchlist
-        # if self.scan is not None:
-        #     self.watchlist  = instrument_list.get_watchlist(self.scan)
-        #     self.scan_results = {}
-        # TODO - validate backtest validation functionality
-        # elif self.instruments is not None:
-        #     self.watchlist  = self.instruments.split(',') 
-        # elif self.validation_file is not None:
-        #     self.raw_watchlist = livetrade_history.Instrument.unique() # FOR OANDA
-        # else:
-        #     self.watchlist  = config["WATCHLIST"]
-        
         self._assign_broker(broker_config)
         self.configure_emailing(global_config)
         
@@ -232,7 +272,7 @@ class AutoTrader():
                 print("--------------------")
                 print("Time: {}".format(datetime.now().strftime("%A, %B %d %Y, "+
                                                                   "%H:%M:%S")))
-                
+        
         ''' -------------------------------------------------------------- '''
         '''    Assign strategy to bot for each instrument in watchlist     '''
         ''' -------------------------------------------------------------- '''
@@ -245,7 +285,7 @@ class AutoTrader():
                         # Send bot to bot manager to monitor stream
                         print("Passing bot to bot manager...")
                         bot_name_string = "{}_{}_{}".format(strategy.replace(' ',''),
-                                                            self.strategies[strategy]['INTERVAL'],
+                                                            self.strategies[strategy]['INTERVAL'].split(',')[0],
                                                             instrument)
                         ManageBot(bot, self.home_dir, bot_name_string)
                     else:
@@ -295,15 +335,6 @@ class AutoTrader():
                              self.broker, starting_balance, self.broker_utils) 
                     self.print_backtest_results(backtest_results)
                     
-                    if self.validation_file is not None:
-                        final_balance_diff = bot.livetrade_results['final_balance_difference']
-                        no_live_trades = bot.livetrade_results['no_live_trades']
-                        
-                        print("\n            Backtest Validation")
-                        print("-------------------------------------------")
-                        print("Difference between final portfolio balance between")
-                        print("live-trade account and backtest is ${}.".format(round(final_balance_diff, 2)))
-                        print("Number of live trades: {} trades.".format(no_live_trades))
                 else:
                     self.multibot_backtest_results = self.multibot_backtest_analysis()
                     self.print_multibot_backtest_results(self.multibot_backtest_results)
@@ -314,13 +345,8 @@ class AutoTrader():
             
             if self.show_plot:
                 if len(self.bots_deployed) == 1:
-                    if self.validation_file is None:
-                        if len(self.bots_deployed[0].backtest_summary['trade_summary']) > 0:
-                            self.plot_backtest(bot=self.bots_deployed[0])
-                        
-                    else:
-                        self.plot_backtest(bot=self.bots_deployed[0], 
-                                           validation_file=self.validation_file)
+                    if len(self.bots_deployed[0].backtest_summary['trade_summary']) > 0:
+                        self.plot_backtest(bot=self.bots_deployed[0])
                 
                 else:
                     # Backtest run with multiple bots
@@ -365,22 +391,30 @@ class AutoTrader():
                 the strategy dictionary can be passed directly.
         '''
         
-        if strategy_dict is None:
-            config_file_path = os.path.join(self.home_dir, 'config', strategy_filename)
-            new_strategy = read_yaml(config_file_path + '.yaml')
+        if self.home_dir is None:
+            # Home directory has not yet been set, postpone strategy addition
+            if strategy_filename is None:
+                self._uninitiated_strat_dicts.append(strategy_dict)
+            else:
+                self._uninitiated_strat_files.append(strategy_filename)
+            
         else:
-            new_strategy = strategy_dict
-        
-        name = new_strategy['NAME']
-        
-        if name in self.strategies:
-            print("Warning: duplicate strategy name deteced. Please check " + \
-                  "the NAME field of your strategy configuration file and " + \
-                  "make sure it is not the same as other strategies being " + \
-                  "run from this instance.")
-            print("Conflicting name:", name)
-        
-        self.strategies[name] = new_strategy
+            if strategy_dict is None:
+                config_file_path = os.path.join(self.home_dir, 'config', strategy_filename)
+                new_strategy = read_yaml(config_file_path + '.yaml')
+            else:
+                new_strategy = strategy_dict
+            
+            name = new_strategy['NAME']
+            
+            if name in self.strategies:
+                print("Warning: duplicate strategy name deteced. Please check " + \
+                      "the NAME field of your strategy configuration file and " + \
+                      "make sure it is not the same as other strategies being " + \
+                      "run from this instance.")
+                print("Conflicting name:", name)
+            
+            self.strategies[name] = new_strategy
     
     
     def backtest(self, start=None, end=None, initial_balance=1000, spread=0, 
@@ -431,6 +465,65 @@ class AutoTrader():
         self.backtest_base_currency = base_currency
     
     
+    def configure(self, feed='yahoo', verbosity=1, notify=0, home_dir=None,
+                  include_broker=False, use_stream=False, detach_bot=False,
+                  check_data_alignment=True, allow_dancing_bears=False,
+                  account_id=None, environment='demo', show_plot=False,
+                  MTF_initialisation=False):
+        '''
+        AutoTrader Run Configuration
+        -------------------------------
+        
+        Configures various run settings for AutoTrader.
+        
+            Parameters:
+                feed (str): the data feed to be used (eg. Yahoo, Oanda).
+                
+                verbosity (int): the verbosity of AutoTrader (0, 1 or 2).
+                
+                notify (int): the level of email notification (0, 1 or 2).
+                
+                home_dir (str): the project home directory.
+                
+                include_broker (bool): set to True to assign broker to strategy
+                attributes.
+                
+                use_stream (bool): set to True to use price stream as data feed.
+                
+                detach_bot (bool): set to True to spawn new thread for each bot
+                deployed.
+                
+                check_data_alignment (bool): verify time of latest candle in
+                data recieved against current time.
+                
+                allow_dancing_bears (bool): allow incomplete candles to be 
+                passed to strategy.
+                
+                account_id (str): the brokerage account ID to use in this instance.
+                
+                environment (str): the trading environment of this instance.
+                
+                show_plot (bool): automatically display plot of results.
+                
+                MTF_initialisation (bool): only download mutliple time frame 
+                data when initialising the strategy, rather than every update.
+        '''
+        
+        self.feed = feed
+        self.verbosity = verbosity
+        self.notify = notify
+        self.home_dir = home_dir if home_dir is not None else os.getcwd()
+        self.include_broker = include_broker
+        self.use_stream = use_stream
+        self.detach_bot = detach_bot
+        self.check_data_alignment = check_data_alignment
+        self.allow_dancing_bears = allow_dancing_bears
+        self.account_id = account_id
+        self.environment = environment
+        self.show_plot = show_plot
+        self.MTF_initialisation = MTF_initialisation
+        
+    
     def scan(self, scan_index=None):
         '''
         Configure AutoTrader scan. 
@@ -454,28 +547,19 @@ class AutoTrader():
         self.scan_index = scan_index
     
     
-    def plot_backtest(self, bot=None, validation_file=None):
+    def plot_backtest(self, bot=None):
         '''
         Plots backtest results of an AutoTrader Bot.
             
             Parameters:
                 bot (class): AutoTrader bot class containing backtest results.
-                
-                validation_file (str): filepath of backtest validation file.
         '''
         ap = autoplot.AutoPlot(bot.data)
         profit_df = pd.merge(bot.data, 
                              bot.backtest_summary['trade_summary']['Profit'], 
                              left_index=True, right_index=True).Profit.cumsum()
         
-        if validation_file is None:
-            ap.plot(bot.backtest_summary, cumulative_PL=profit_df)
-            
-        else:
-            ap.plot_validation_balance = self.plot_validation_balance # User option flag
-            ap.ohlc_height = 350
-            ap.validate_backtest(bot.livetrade_results['summary'],
-                                 bot.backtest_summary)
+        ap.plot(bot.backtest_summary, cumulative_PL=profit_df)
                 
     
     def multibot_backtest_analysis(self, bots=None):
@@ -686,13 +770,6 @@ class AutoTrader():
                     banner = pyfiglet.figlet_format("AutoBacktest")
                     print(banner)
                 
-                if self.validation_file is not None:
-                    # Also get broker-specific utility functions
-                    validation_utils = importlib.import_module('autotrader.brokers.{}.utils'.format(self.feed.lower()))
-                    
-                    # Correct watchlist
-                    if self.instruments is None:
-                        self.watchlist = validation_utils.format_watchlist(self.raw_watchlist)
                 
         else:
             utils_module    = importlib.import_module('autotrader.brokers.{}.utils'.format(self.feed.lower()))
