@@ -124,13 +124,21 @@ def sma(data, period=14):
 
 
 def crossover(list_1, list_2):
-    ''' Returns a list of length len(list_1) with 1 when list_1 crosses above
+    ''' 
+    Returns a list of length len(list_1) with 1 when list_1 crosses above
     list_2 and -1 when list_1 crosses below list_2.
-    
     '''
-
-    difference = [list_1[i] - list_2[i] for i in range(len(list_1))]
-    sign_list = np.where(np.sign(difference) < 0, -1,1)
+    
+    sign_list = []
+    for i in range(len(list_1)):
+        if np.isnan(list_1[i]):
+            sign_list.append(np.nan)
+        else:
+            difference = list_1[i] - list_2[i]
+            if difference < 0:
+                sign_list.append(-1)
+            else:
+                sign_list.append(1)
     
     crossover_list = [0]
     
@@ -145,7 +153,6 @@ def crossover(list_1, list_2):
     return crossover_list
 
 
-
 def cross_values(a, b, ab_crossover):
     cross_point_list = [0]
     last_cross_point = 0
@@ -383,11 +390,12 @@ def find_swings(data, use_body = False):
     
     grad    = np.gradient(ema)
     
-    
-    zeros   = np.where(np.sign(grad[1:len(grad)]) != np.sign(grad[0:-1]), 1, 0)
-    zeros   = np.insert(zeros, 0, 0)
-    swings  = -zeros*np.sign(grad)
-    
+    swings   = np.zeros(len(grad))
+    for i in range(1, len(grad)):
+        if not np.isnan(grad[i-1]):
+            if np.sign(grad[i]) != np.sign(grad[i-1]):
+                swings[i] = -np.sign(grad[i])
+
     swing_df = pd.DataFrame(data=swings, index=data.index, columns=['swing'])
     
     low_list    = []
@@ -591,6 +599,80 @@ def last_level_crossed(data, base):
     
     return levels_crossed
 
+
+def build_multiplier_grid(origin, direction, multiplier, no_levels, precision, spacing):
+    '''
+    Constructs grid levels with a multiplying grid space.
+    
+        Parameters:
+            origin (float): origin of grid as price amount.
+            
+            direction (int): direction of grid (1 for long, -1 for short).
+            
+            multiplier (float): grid space multiplier when price moves away 
+            from the origin opposite to direction.
+            
+            no_levels (int): number of levels to calculate either side of the 
+            origin.
+            
+            precision (int): instrument precision (eg. 4 for most currencies, 2 
+            for JPY).
+            
+            spacing (float): spacing of grid in price units.
+    '''
+    
+    levels = [i for i in range(1, no_levels + 1)]
+
+    pos_levels = [round(origin + direction*spacing*i, precision) for i in levels]
+    neg_spaces = [spacing*multiplier**(i-1) for i in levels]
+    neg_levels = []
+    prev_neg_level = origin
+    for i in range(len(levels)):
+        next_neg_level = prev_neg_level - direction*neg_spaces[i]
+        prev_neg_level = next_neg_level
+        neg_levels.append(round(next_neg_level, precision))
+    
+    grid = neg_levels + [origin] + pos_levels
+    grid.sort()
+    
+    return grid
+
+
+def last_level_touched(data, grid):
+    '''
+    Calculates the grid levels touched by price data.
+    '''
+    
+    # initialise with nan
+    last_level_crossed = np.nan 
+    
+    levels_touched = []
+    for i in range(len(data)):
+        high = data.High.values[i]
+        low = data.Low.values[i]
+        
+        upper_prices = []
+        lower_prices = []
+        
+        for price in [high, low]:    
+            # Calculate level above
+            upper_prices.append(grid[next(x[0] for x in enumerate(grid) if x[1] > price)])
+            
+            # calculate level below
+            first_level_below_index = next(x[0] for x in enumerate(grid[::-1]) if x[1] < price)
+            lower_prices.append(grid[-(first_level_below_index+1)])
+        
+        if lower_prices[0] != lower_prices[1]:
+            # Candle has crossed a level, since the level below the candle high
+            # is different to the level below the candle low.
+            # This essentially means the grid level is between candle low and high.
+            last_level_crossed = lower_prices[0]
+        
+        levels_touched.append(last_level_crossed)
+    
+    return levels_touched
+
+
 # def TDI(data):
 #     rsiPeriod = input(11, minval = 1, title = "RSI Period")
 #     bandLength = input(31, minval = 1, title = "Band Length")
